@@ -1,8 +1,9 @@
 /*
 	pam_shield_lib.c	WJ106
 
-    pam_shield 0.9.2 WJ107
-    Copyright (C) 2007  Walter de Jong <walter@heiho.net>
+    pam_shield 0.9.6 WJ107
+    Copyright (C) 2007-2012  Walter de Jong <walter@heiho.net>
+    and Jonathan Niehof <jtniehof@gmail.com>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,12 +19,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-/*
-	This file is not a true library; this code is #include'd into
-	the pam_shield sources. The reason for this is that I had problems
-	with having lots of duplicate code, while the symbols should be 'static'
-	in the resulting pam_shield.so shared library
-*/
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,47 +39,30 @@
 #include <errno.h>
 #include <gdbm.h>
 
-#define PAM_SHIELD_VERSION		"0.9.2"
+#include "pam_shield_lib.h"
 
-#define DEFAULT_CONFFILE		"/etc/security/shield.conf"
-#define DEFAULT_DBFILE			"/var/lib/pam_shield/db"
-#define DEFAULT_TRIGGER_CMD		"/usr/sbin/pam_shield-trigger"
-#define DEFAULT_MAX_CONNS		10
-#define DEFAULT_INTERVAL		60L
-#define DEFAULT_RETENTION		(3600L * 24L)
+#pragma GCC visibility push(hidden)
 
-#define MAX_LINE				1024
+int options = 0;
+GDBM_FILE dbf;
 
-#define OPT_DEBUG				1
-#define OPT_BLOCK_ALL			2		/* block all, including known users */
-#define OPT_DRYRUN				4
-#define OPT_LISTDB				8
-#define OPT_MISSING_DNS			0x10	/* allow missing DNS */
-#define OPT_MISSING_REVERSE		0x20	/* allow missing reverse DNS */
-
-static int options = 0;
-static GDBM_FILE dbf;
-
-static char *conffile = NULL;
-static char *dbfile = NULL;
-static char *trigger_cmd = NULL;
+char *conffile = NULL;
+char *dbfile = NULL;
+char *trigger_cmd = NULL;
 
 /* white lists of addresses */
-static ip_list *allow_ipv4_list = NULL;
-static ip_list *allow_ipv6_list = NULL;
-static name_list *allow_names = NULL;
+ip_list *allow_ipv4_list = NULL;
+ip_list *allow_ipv6_list = NULL;
+name_list *allow_names = NULL;
 
-static int max_conns = DEFAULT_MAX_CONNS;
-static long interval = DEFAULT_INTERVAL;
-static long retention = DEFAULT_RETENTION;
+int max_conns = DEFAULT_MAX_CONNS;
+long interval = DEFAULT_INTERVAL;
+long retention = DEFAULT_RETENTION;
 
-static time_t this_time;
+time_t this_time;
 
 
-static void logmsg(int level, const char *fmt, ...);
-
-
-static ip_list *new_ip_list(void) {
+ip_list *new_ip_list(void) {
 ip_list *ip;
 
 	if ((ip = (ip_list *)malloc(sizeof(ip_list))) == NULL)
@@ -94,7 +72,7 @@ ip_list *ip;
 	return ip;
 }
 
-static void destroy_ip_list(ip_list *list) {
+void destroy_ip_list(ip_list *list) {
 ip_list *p;
 
 	while(list != NULL) {
@@ -104,7 +82,7 @@ ip_list *p;
 	}
 }
 
-static void add_ip_list(ip_list **root, ip_list *ip) {
+void add_ip_list(ip_list **root, ip_list *ip) {
 	if (root == NULL || ip == NULL)
 		return;
 
@@ -134,7 +112,7 @@ static void add_ip_list(ip_list **root, ip_list *ip) {
 	try to match an IP number against the allow list
 	returns 1 if it matches
 */
-static int match_ipv4_list(unsigned char *saddr) {
+int match_ipv4_list(unsigned char *saddr) {
 ip_list *ip;
 int i, match;
 
@@ -158,7 +136,7 @@ int i, match;
 	return 0;
 }
 
-static int match_ipv6_list(unsigned char *saddr) {
+int match_ipv6_list(unsigned char *saddr) {
 ip_list *ip;
 int i, match;
 
@@ -185,7 +163,7 @@ int i, match;
 /*
 	name_lists are hostnames and/or network names
 */
-static name_list *new_name_list(char *name) {
+name_list *new_name_list(char *name) {
 name_list *n;
 
 	if (name == NULL || !*name)
@@ -199,7 +177,7 @@ name_list *n;
 	return n;
 }
 
-static void destroy_name_list(name_list *list) {
+void destroy_name_list(name_list *list) {
 name_list *p;
 
 	while(list != NULL) {
@@ -209,7 +187,7 @@ name_list *p;
 	}
 }
 
-static void add_name_list(name_list **root, name_list *n) {
+void add_name_list(name_list **root, name_list *n) {
 	if (root == NULL || n == NULL)
 		return;
 
@@ -227,11 +205,12 @@ static void add_name_list(name_list **root, name_list *n) {
 	*root = n;
 }
 
+
 /*
 	see if 'name' matches our whitelist
 	return 1 if it does
 */
-static int match_name_list(char *name) {
+int match_name_list(char *name) {
 name_list *n;
 
 	if (name == NULL || !*name)
@@ -257,7 +236,7 @@ name_list *n;
 /*
 	initialize variables
 */
-static int init_module(void) {
+int init_module(void) {
 	this_time = time(NULL);
 
 	conffile = strdup(DEFAULT_CONFFILE);
@@ -271,7 +250,7 @@ static int init_module(void) {
 	return 0;
 }
 
-static void deinit_module(void) {
+void deinit_module(void) {
 	if (conffile != NULL) {
 		free(conffile);
 		conffile = NULL;
@@ -297,7 +276,7 @@ static void deinit_module(void) {
 /*
 	strip leading and trailing whitespace from a string
 */
-static void strip(char *str) {
+void strip(char *str) {
 char *p;
 int i;
 
@@ -317,6 +296,7 @@ int i;
 		str[i--] = 0;
 }
 
+
 /*
 	multipliers:
 		1s	second
@@ -330,7 +310,7 @@ int i;
 	default is 1
 	returns 0 on error
 */
-static long get_multiplier(char *str) {
+long get_multiplier(char *str) {
 	if (str == NULL || !*str)
 		return 1L;
 
@@ -369,7 +349,7 @@ static long get_multiplier(char *str) {
 	mask is struct in_addr.saddr, size is the size of the array
 	(4 for IPv4, 16 for IPv6)
 */
-static void ip_bitmask(int bits, unsigned char *mask, int size) {
+void ip_bitmask(int bits, unsigned char *mask, int size) {
 int i, num, rest;
 
 	if (mask == NULL)
@@ -400,7 +380,7 @@ int i, num, rest;
 	allow network/netmask, for both IPv4 and IPv6
 	netmask can be in canonical or decimal notation
 */
-static int allow_ip(char *ipnum, int line_no) {
+int allow_ip(char *ipnum, int line_no) {
 char *netmask;
 ip_list *ip;
 name_list *name;
@@ -502,8 +482,9 @@ int bits;
 /*
 	read configuration file
 */
-static int read_config(void) {
+int read_config(void) {
 FILE *f;
+struct stat statbuf;
 char buf[MAX_LINE], *p, *endp;
 int line_no, err;
 long multiplier;
@@ -624,6 +605,10 @@ long multiplier;
 				logmsg(LOG_CRIT, "out of memory");
 				err--;
 			}
+			if (stat(trigger_cmd, &statbuf) == -1) {
+				logmsg(LOG_ALERT, "%s:%d: command '%s' not found", conffile, line_no, trigger_cmd);
+				err--;
+			}
 			continue;
 		}
 		if (!strcmp(buf, "max_conns")) {
@@ -669,7 +654,7 @@ long multiplier;
 	print the IP number of a db_record
 	return NULL on error, or buf on success
 */
-static const char *print_ip(_pam_shield_db_rec_t *record, char *buf, int buflen) {
+const char *print_ip(_pam_shield_db_rec_t *record, char *buf, int buflen) {
 	if (buf == NULL || buflen <= 1)
 		return NULL;
 
@@ -696,7 +681,7 @@ static const char *print_ip(_pam_shield_db_rec_t *record, char *buf, int buflen)
 /*
 	run external command
 */
-static int run_trigger(char *cmd, _pam_shield_db_rec_t *record) {
+int run_trigger(char *cmd, _pam_shield_db_rec_t *record) {
 char ipbuf[INET6_ADDRSTRLEN];
 pid_t pid;
 
@@ -730,13 +715,17 @@ pid_t pid;
 		exit(-1);
 	} else {
 		pid_t err;
+		int status;
 
-		while((err = waitpid(pid, NULL, 0)) > 0);
+		while((err = waitpid(pid, &status, 0)) > 0);
+
+		if (WEXITSTATUS(status) != 0)
+			return -1;
 	}
 	return 0;
 }
 
-static int expire_record(_pam_shield_db_rec_t *record) {
+int expire_record(_pam_shield_db_rec_t *record) {
 int updated;
 char ipbuf[INET6_ADDRSTRLEN];
 
@@ -772,7 +761,7 @@ char ipbuf[INET6_ADDRSTRLEN];
 /*
 	gdbm has encountered a fatal error
 */
-static void fatal_func(char *str) {
+void fatal_func(char *str) {
 	logmsg(LOG_ERR, "gdbm encountered a fatal error : %s; resetting the database", str);
 
 	gdbm_close(dbf);
@@ -780,4 +769,5 @@ static void fatal_func(char *str) {
 		logmsg(LOG_ERR, "failed to create new gdbm file '%s' : %s", dbfile, gdbm_strerror(gdbm_errno));
 }
 
+#pragma GCC visibility pop
 /* EOB */
